!file StreamScenariosAug.f90

program StreamScenariosAug


! source code notes

! Compiling with Gnu Fortran:
! Prior to compiling, linking or running:
!   set path=%path%;C:\MinGW\bin\
! To compile:
!   Switches: -c: Compile without linking; -o link without compiling; -ffree-line-length-none (free format source code, no line length limit), -Wall (Warnings all)
!   gfortran  -c -Wall -Wextra -fcheck=bounds -Fbacktrace -ffree-line-length-none StreamScenariosAug.f90
!   gfortran -static -o StreamScenariosAug StreamScenariosAug.o

IMPLICIT NONE

integer, parameter :: mxseg=5705
!  integer, parameter :: mxper=817

!  integer, parameter :: mxrow=180
!  integer, parameter :: mxcol=335
!  integer, parameter :: nodes=mxrow*mxcol

character*128 FILE
integer IARGC,IOS

CHARACTER*128 BUFF,SFRIN,SFROUT
character*128 ch_scenario
character*80 strm_names,header
! character*128 scenario_file,

integer nper, id_scenario, iper, i, istrm, numseg

!Declare input data:

!1. Data: NSTRM NSS NSFRPAR NPARSEG CONST DLEAK ISTCB1 ISTCB2
integer nstrm,nss,nsfrpar,nparseg,istcb1,istcb2
real const,dleak

!One record for each stream reach:
!2. Data: KRCH IRCH JRCH ISEG IREACH RCHLEN
integer krch,irch,jrch,iseg,ireach
real rchlen
character*60 chstrm(mxseg)
integer iaug(mxseg)

!FOR EACH STRESS PERIOD:
!5. Data: ITMP IRDFLG IPTFLG {NP}
integer itmp,irdflg,iptflg
! integer np

!6a. Data: NSEG ICALC OUTSEG IUPSEG {IPRIOR} {NSTRPTS} FLOW RUNOFF ETSW PPTSW {ROUGHCH} {ROUGHBK} {CDPTH} {FDPTH} {AWDTH} {BWDTH}
integer nseg,icalc,outseg,iupseg
integer iprior              !,nstrpts
real flow,runoff,etsw,pptsw
! real roughch,roughbk,cdpth,fdpth,awdth,bwdth

!6b. Data: HCOND1 THICKM1 ELEVUP {WIDTH1} {DEPTH1}  [data for upstream end]
real hcond1,thickm1,elevup,width1,depth1

!6c. Data: HCOND2 THICKM2 ELEVDN {WIDTH2} {DEPTH2}  [data for downstream end]
real hcond2,thickm2,elevdn,width2,depth2

!6a-c (no change in data):
character*256 line

! declarations for debugging:
! integer nperdbg, iperprt(10),iperdbg(mxper)

! declarations for augmentation flow:
integer numaug              !number of stream segments where augmentation inflow is specified (begin with 1 or 0)
integer segaug              !segment to which augmentation flow is to be added as inflow
integer monaug(12)          !monthly augmentation pattern (1 or 0)
real    augflow(1939:2007)  !augmentation flow each year (cfs)
real    Qaug                !augmentation flow to each segment in a particular year (cfd)
integer augon               !indicate that aug flow is on in current stress period (1 or 0)

integer iyr,mo,j,idx
character*2 chmo(12)
character*7 chper
real sec_day

data chmo /'01','02','03','04','05','06','07','08','09','10','11','12'/

!--------------------------------------------------------------------
sec_day = 86400.            !sec/day

if (IARGC().ne.1) then
  write(*,*) 'Usage: StreamScenarios <parameter file>'
  STOP
endif
call GETARG(1,FILE)
write(*,*) 'Open '//trim(FILE)
OPEN(1,FILE=FILE,STATUS='OLD',IOSTAT=IOS)
if (IOS.ne.0) then
  write(*,*) 'Error opening parameter file.'
  STOP
endif

READ(1,*) SFRIN
READ(1,*) SFROUT
read(1,*) strm_names
read(1,*) numseg,nper                  !,nperdbg
!                                       read(1,*) (iperdbg(j),j=1,nperdbg)
read(1,*) id_scenario,ch_scenario
read(1,*) numaug

monaug=0
augflow=0
segaug=0
Qaug=0
iaug=0   !initialize array iaug(mxseg)

if (numaug.gt.0) then
!   define monthly and annual augmentation pumping patterns:
  read(1,*) (monaug(j),j=1,12)          !1 or 0 (on or off)
  read(1,*) (augflow(j),j=1939,2007)    !cfs
!   define stream segment to which augmentation pipeflow is to be added as inflow
  read(1,*) segaug
  iaug(segaug)=1      !set indicator for augmentation flow for segment segaug
endif
CLOSE(1)

write(*,*) 'numseg = ',numseg,' nper = ',nper
write(*,*) 'Scenario ',id_scenario,': '//trim(ch_scenario)
write(*,*) 'numaug=',numaug,' segaug=',segaug
write(*,'(1x,a12,1x,12i2)') 'monaug = ',monaug
write(*,*) 'augflow (cfs) 1939-2007:'
write(*,'(10f8.2)') augflow

open(1,file=trim(strm_names))
read(1,*) header
do idx=1,numseg
  read(1,'(a)') chstrm(idx)
enddo
close(1)

open(1,file=trim(SFRIN))
open(2,file=trim(SFROUT))
!0:
do i=1,2
  read (1,'(a)') buff
  write(2,'(a)') buff
  write(*,'(1x,a)') buff
enddo
!1: for each simulation:
read (1,*) nstrm,nss,nsfrpar,nparseg,const,dleak,istcb1,istcb2
write(2,'(4i5,2f10.5,2i5)') nstrm,nss,nsfrpar,nparseg,const,dleak,istcb1,istcb2
write(*,'(1x,4i5,2f10.5,2i5)') nstrm,nss,nsfrpar,nparseg,const,dleak,istcb1,istcb2
!2: One record for each stream reach:
do istrm=1,nstrm
  read (1,*) krch,irch,jrch,iseg,ireach,rchlen
  write(2,'(5i5,f8.1,1x,a)') krch,irch,jrch,iseg,ireach,rchlen,trim(chstrm(istrm))
enddo

iyr=1939
mo=11
Qaug=0 !initialize array to specify inflow to top reach of each segment

!FOR EACH STRESS PERIOD:
do iper=1,nper

!     Increment modulo month counter mo; on mo=1 increment year and initialize annual sums.
  mo=mod(mo,12)+1
  if(mo.eq.1) then
    iyr=iyr+1
  endif
  write(chper,'(a2,1h/,i4)') chmo(mo),iyr
  
  if (numaug.gt.0 .and. monaug(mo).eq.1 .and. augflow(iyr).gt.1e-6) then
    augon = 1
    Qaug = sec_day*augflow(iyr)   !convert flow from cfs to cfd
    ! write(*,*) chper//' segaug=',segaug,' Qaug=',Qaug
  else
    augon = 0
    Qaug=0
  endif
  if (iper.eq.1) write(*,'(1x,a)') &
    'mo/year nseg         FLOW_cfd        Qaug_cfd        Qaug_cfs stream_name'
!5:
  read (1,*) itmp,irdflg,iptflg
  if (iper.eq.1) then
    write(2,'(3i5,30x,a)') itmp,irdflg,iptflg,chper//' sp=predev'
  else
    write(2,'(3i5,30x,a,i5)') itmp,irdflg,iptflg,chper//' sp=',iper-1
  endif

  if (itmp.gt.0) then
    do istrm=1,itmp

      if (augon.eq.0.or.istrm.ne.segaug) then
!6a-c (no change):
        do idx=1,3
          read(1,'(a)') line
          write(2,'(a)') trim(line)
        enddo
      else
!6a:
        read (1,*) nseg,icalc,outseg,iupseg,FLOW,RUNOFF,ETSW,PPTSW
        if (iupseg.gt.0) then
          backspace(1,iostat=ios)
          if(ios.ne.0) then
            write(*,*) 'Error iostat=',ios,' on backspace at istrm=',istrm,' nseg=',nseg
            STOP
          endif
          read (1,*) nseg,icalc,outseg,iupseg,iprior,FLOW,RUNOFF,ETSW,PPTSW
        endif

        FLOW = FLOW + Qaug       !add aug flow to segment's top reach

        if (iupseg.gt.0) then
          write (2,'(5i5,2f16.2,2F13.7)') nseg,icalc,outseg,iupseg,iprior,FLOW,RUNOFF,ETSW,PPTSW
        else
          write(2,'(4i5,2f16.2,2F13.7)') nseg,icalc,outseg,iupseg,FLOW,RUNOFF,ETSW,PPTSW
        endif
        if (FLOW.gt.0.or.Qaug.gt.0) write(*,'(1x,a,i5,1x,3f16.2,1x,a)') &
          chper,nseg,FLOW,Qaug,Qaug/sec_day,trim(chstrm(istrm))
!6b:
        read (1,*) hcond1,thickm1,elevup,width1,depth1
        write(2,'(f10.6,f6.2,f10.3,2f10.2)') hcond1,thickm1,elevup,width1,depth1
!6c:
        read (1,*) hcond2,thickm2,elevdn,width2,depth2
        write(2,'(f10.6,f6.2,f10.3,2f10.2)') hcond2,thickm2,elevdn,width2,depth2

      endif

    enddo  !istrm=1,itmp
  endif
enddo  !iper=1,nper

CLOSE(1)
CLOSE(2)

STOP
END
